@@ -10,108 +10,6 @@
 ;;; separately for each Lisp. Each is declared as a generic function
 ;;; for which swank-<implementation>.lisp provides methods.
 
-(defpackage swank/backend
-  (:use cl)
-  (:nicknames swank-backend)
-  (:export *debug-swank-backend*
-           sldb-condition
-           compiler-condition
-           original-condition
-           message
-           source-context
-           condition
-           severity
-           with-compilation-hooks
-           make-location
-           location
-           location-p
-           location-buffer
-           location-position
-	   location-hints
-           position-p
-           position-pos
-           print-output-to-string
-           quit-lisp
-           references
-           unbound-slot-filler
-           declaration-arglist
-           type-specifier-arglist
-           with-struct
-           when-let
-	   defimplementation
-	   converting-errors-to-error-location
-	   make-error-location
-	   deinit-log-output
-           ;; interrupt macro for the backend
-           *pending-slime-interrupts*
-           check-slime-interrupts
-           *interrupt-queued-handler*
-           ;; inspector related symbols
-           emacs-inspect
-           label-value-line
-           label-value-line*
-           with-symbol
-           ;; package helper for backend
-           import-to-swank-mop
-           import-swank-mop-symbols
-	   ;;
-
-           ))
-
-;; FIXME: rename to sawnk/mop
-(defpackage swank-mop
-  (:use)
-  (:export
-   ;; classes
-   standard-generic-function
-   standard-slot-definition
-   standard-method
-   standard-class
-   eql-specializer
-   eql-specializer-object
-   ;; standard-class readers
-   class-default-initargs
-   class-direct-default-initargs
-   class-direct-slots
-   class-direct-subclasses
-   class-direct-superclasses
-   class-finalized-p
-   class-name
-   class-precedence-list
-   class-prototype
-   class-slots
-   specializer-direct-methods
-   ;; generic function readers
-   generic-function-argument-precedence-order
-   generic-function-declarations
-   generic-function-lambda-list
-   generic-function-methods
-   generic-function-method-class
-   generic-function-method-combination
-   generic-function-name
-   ;; method readers
-   method-generic-function
-   method-function
-   method-lambda-list
-   method-specializers
-   method-qualifiers
-   ;; slot readers
-   slot-definition-allocation
-   slot-definition-documentation
-   slot-definition-initargs
-   slot-definition-initform
-   slot-definition-initfunction
-   slot-definition-name
-   slot-definition-type
-   slot-definition-readers
-   slot-definition-writers
-   slot-boundp-using-class
-   slot-value-using-class
-   slot-makunbound-using-class
-   ;; generic function protocol
-   compute-applicable-methods-using-classes
-   finalize-inheritance))
-
 (in-package swank/backend)
 
 
@@ -129,6 +27,8 @@ magic but really show every frame including SWANK related ones.")
   "List of interface functions that are not implemented.
 DEFINTERFACE adds to this list and DEFIMPLEMENTATION removes.")
 
+(defvar *log-output* nil)            ; should be nil for image dumpers
+
 (defmacro definterface (name args documentation &rest default-body)
   "Define an interface function for the backend to implement.
 A function is defined with NAME, ARGS, and DOCUMENTATION.  This
@@ -248,13 +148,19 @@ This will be used like so:
   `(let ((,var ,value))
      (when ,var ,@body)))
 
-(defun with-symbol (name package)
-  "Generate a form suitable for testing with #+."
-  (if (and (find-package package)
-           (find-symbol (string name) package))
+(defun boolean-to-feature-expression (value)
+  "Converts a boolean VALUE to a form suitable for testing with #+."
+  (if value
       '(:and)
       '(:or)))
 
+(defun with-symbol (name package)
+  "Check if a symbol with a given NAME exists in PACKAGE and returns a
+form suitable for testing with #+."
+  (boolean-to-feature-expression
+   (and (find-package package)
+        (find-symbol (string name) package))))
+
 
 ;;;; UFT8
 
@@ -570,6 +476,21 @@ This is used to resolve filenames without directory component."
   "Return a suitable initial value for SWANK:*READTABLE-ALIST*."
   '())
 
+
+;;;; Packages
+
+(definterface package-local-nicknames (package)
+  "Returns an alist of (local-nickname . actual-package) describing the
+nicknames local to the designated package."
+  (declare (ignore package))
+  nil)
+
+(definterface find-locally-nicknamed-package (name base-package)
+  "Return the package whose local nickname in BASE-PACKAGE matches NAME.
+Return NIL if local nicknames are not implemented or if there is no
+such package."
+  (cdr (assoc name (package-local-nicknames base-package) :test #'string-equal)))
+
 
 ;;;; Compilation
 
@@ -786,7 +707,7 @@ available."
         (and (consp form) (length=2 form)
              (eq (first form) 'setf) (symbolp (second form))))))
 
-(definterface macroexpand-all (form)
+(definterface macroexpand-all (form &optional env)
    "Recursively expand all macros in FORM.
 Return the resulting form.")
 
@@ -814,6 +735,50 @@ NIL."
                    (values new-form expanded)))))
     (frob form env)))
 
+(defmacro with-collected-macro-forms
+    ((forms &optional result) instrumented-form &body body)
+  "Collect macro forms by locally binding *MACROEXPAND-HOOK*.
+
+Evaluates INSTRUMENTED-FORM and collects any forms which undergo
+macro-expansion into a list.  Then evaluates BODY with FORMS bound to
+the list of forms, and RESULT (optionally) bound to the value of
+INSTRUMENTED-FORM."
+  (assert (and (symbolp forms) (not (null forms))))
+  (assert (symbolp result))
+  (let ((result-symbol (or result (gensym))))
+   `(call-with-collected-macro-forms
+     (lambda (,forms ,result-symbol)
+       (declare (ignore ,@(and (not result)
+                               `(,result-symbol))))
+       ,@body)
+     (lambda () ,instrumented-form))))
+
+(defun call-with-collected-macro-forms (body-fn instrumented-fn)
+  (let ((return-value nil)
+        (collected-forms '()))
+    (let* ((real-macroexpand-hook *macroexpand-hook*)
+           (*macroexpand-hook*
+            (lambda (macro-function form environment)
+              (let ((result (funcall real-macroexpand-hook
+                                     macro-function form environment)))
+                (unless (eq result form)
+                  (push form collected-forms))
+                result))))
+      (setf return-value (funcall instrumented-fn)))
+    (funcall body-fn collected-forms return-value)))
+
+(definterface collect-macro-forms (form &optional env)
+  "Collect subforms of FORM which undergo (compiler-)macro expansion.
+Returns two values: a list of macro forms and a list of compiler macro
+forms."
+  (with-collected-macro-forms (macro-forms expansion)
+      (ignore-errors (macroexpand-all form env))
+    (with-collected-macro-forms (compiler-macro-forms)
+        (handler-bind ((warning #'muffle-warning))
+          (ignore-errors
+            (compile nil `(lambda () ,expansion))))
+      (values macro-forms compiler-macro-forms))))
+
 (definterface format-string-expand (control-string)
   "Expand the format string CONTROL-STRING."
   (macroexpand `(formatter ,control-string)))
@@ -1492,8 +1457,7 @@ COMPLETION-FUNCTION, if non-nil, should be called after saving the image.")
 
 (defun deinit-log-output ()
   ;; Can't hang on to an fd-stream from a previous session.
-  (setf (symbol-value (find-symbol "*LOG-OUTPUT*" 'swank))
-        nil))
+  (setf *log-output* nil))
 
 
 ;;;; Wrapping
